# -*- coding: utf-8 -*-
"""
    MoinMoin - WordML Formatter

    @copyright: 2006 by Oleg Kobchenko <olegyk@yahoo.com>

    based on DocBook formatter
    @license: GNU GPL, see COPYING for details.
"""

import sys, traceback, os

from MoinMoin.formatter.base import FormatterBase
from MoinMoin import wikiutil, config
from MoinMoin.error import CompositeError
from MoinMoin.action import AttachFile

from xml.sax import saxutils
from xml.dom import getDOMImplementation

from xml.dom import getDOMImplementation

from MoinMoin.formatter.xml_docbook import Formatter as DocBookFormatter

# ------------------------------------------------------------------------------------
# Configuration -- platform-specfic

wordml_path = "C:/Moin/local/libxml2/WordML"
wordml_template = 'file:///%s/template.xml' % wordml_path
xsltproc_cmd = 'xsltproc.exe --nonet --stringparam template "%s" "%s/wordml.xsl" -' % ('%s', wordml_path)

# Add the following to moin.cgi
# for xsltproc used by WordML formatter
# os.environ['PATH'] = '%s;C:\\Moin\\local\\libxml2\\bin' % os.environ['PATH']

# ------------------------------------------------------------------------------------

class FilterIOError(CompositeError): pass

def execFilterIO(cmd, filename, input):
    """ use cmd to get plaintext content of filename
        to decode to unicode, we use the first coding of codings list that
        does not throw an exception or force ascii
    """
    child_in, child_out, child_err = os.popen3(cmd % filename, "b")

    child_in.write(input)
    child_in.close()
    output = child_out.read()
    status = child_out.close()
    err = child_err.read()
    child_err.close()
    if status or len(err):
        raise FilterIOError("MSWord processor failed: %s" % err)
    
    return output

class WordMLOutputFormatter:
    """
       Format WordML output
    """
    
    def __init__(self, dommDoc):
        self.buffer = []
        self.doc = dommDoc
        self.curNode = dommDoc.documentElement
        
    def setHeading(self, headNode):
        self.domHeadNode = headNode
        return u""
    
    def _printNode(self, node):
        """
            Function print a node
        """
        from xml.dom.ext import Print
        import StringIO
        from xml.dom.ext import Printer
        
        stream = StringIO.StringIO()
        
        visitor = Printer.PrintVisitor(stream, 'UTF-8')
        Printer.PrintWalker(visitor, node).run()
        # get value from stream
        ret = stream.getvalue()
        stream.close()
        
        return unicode(ret, 'utf-8')

    def getHeading(self):
        # return heading from model
        rootNode = self.doc.documentElement
        # print article info
        self.buffer.append('<?xml version="1.0"?><%s>%s' % (rootNode.nodeName,
                                                self._printNode(self.domHeadNode)))
        return u""
        
    def getBody(self):
        body = []
        # print all nodes inside dom behind heading
        firstNode = self.doc.documentElement.firstChild
        while firstNode:
            if firstNode != self.domHeadNode:
                body.append(self._printNode(firstNode))
            firstNode = firstNode.nextSibling
        self.buffer.append(''.join(body))
        return u""

    def getEndContent(self):
        # close all opened tags
        ret = []
        while self.curNode != self.doc.documentElement:
            ret.append("</%s>" % (self.curNode.nodeName, ))
            self.curNode = self.curNode.parentNode
        self.buffer.append(''.join(ret))
        return u""
        
    def getFooter(self):
        self.buffer.append("</%s>" % self.doc.documentElement.nodeName)
        buf = ''.join(self.buffer)
        # --- XSLT ---
        ret = execFilterIO(xsltproc_cmd, wordml_template, buf)
        return ret

class Formatter(DocBookFormatter):
    """
        Send plain text data.
    """

    def __init__(self, request, **kw):
        '''We should use this for creating the doc'''
        DocBookFormatter.__init__(self, request, **kw)

        self.outputFormatter = WordMLOutputFormatter(self.doc)
